#include <math.h>
#include "postgres.h"
#include "funcapi.h"
#include "miscadmin.h"
#include "executor/spi.h"
#include "access/relscan.h"
#include "access/table.h"
#include "access/tableam.h"
#include "access/heapam.h"
#include "nodes/makefuncs.h"
#include "utils/snapmgr.h"

#ifdef PG_MODULE_MAGIC
PG_MODULE_MAGIC;
#endif

PG_FUNCTION_INFO_V1(norm_spi);
PG_FUNCTION_INFO_V1(norm_core);

Datum norm_spi(PG_FUNCTION_ARGS)
{
	double result = 0.0;
	bool is_null;

	char const* query = PG_GETARG_CSTRING(0);
	SPIPlanPtr plan;
    Portal portal;

	SPI_connect();
	plan = SPI_prepare(query, 0, NULL);
    portal = SPI_cursor_open(NULL, plan, NULL, NULL, true);

	while (true)
	{
		SPI_cursor_fetch(portal, true, 1);
		if (SPI_processed) {
            HeapTuple spi_tuple = SPI_tuptable->vals[0];
			Datum val = SPI_getbinval(spi_tuple, SPI_tuptable->tupdesc, 1, &is_null);
			double x = DatumGetFloat8(val);
			result += x*x;
			SPI_freetuple(spi_tuple);
            SPI_freetuptable(SPI_tuptable);
		} else
			break;
	}

    SPI_cursor_close(portal);
 	SPI_finish();

	PG_RETURN_FLOAT8(sqrt(result));
}

Datum norm_core(PG_FUNCTION_ARGS)
{
	double result = 0.0;
	bool is_null;
	RangeVar   *rv = makeRangeVar(NULL, "big", -1);
	Relation rel = heap_openrv_extended(rv, AccessShareLock, false);
	Snapshot snapshot = RegisterSnapshot(GetLatestSnapshot());
	TableScanDesc scan = table_beginscan(rel, snapshot, 0, NULL);
	HeapTuple	tup;
	while ((tup = heap_getnext(scan, ForwardScanDirection)) != NULL)
	{
		Datum val = heap_getattr(tup, 2, RelationGetDescr(rel), &is_null);
		double x = DatumGetFloat8(val);
		result += x*x;
	}
	table_endscan(scan);
	UnregisterSnapshot(snapshot);
	table_close(rel, NoLock);

	PG_RETURN_FLOAT8(sqrt(result));
}
